﻿using System;
using System.Reflection;
using System.Windows;
using System.Windows.Input;
using PostSharp.Patterns.Xaml;

namespace TelerikRadGridViewProblems
{
	/// <summary>
	/// Behavior that will connect an UI event to a viewmodel Command,
	/// allowing the event arguments to be passed as the CommandParameter.
	/// </summary>
	/// <remarks>
	/// Comes from https://stackoverflow.com/a/16317999
	/// </remarks>
	public class EventToCommandBehavior : BehaviorBase<FrameworkElement>
	{
		Delegate _handler;
		EventInfo _oldEvent;

		[DependencyProperty]
		public string Event { get; set; }

		[DependencyProperty]
		public ICommand Command { get; set; }

		[DependencyProperty]
		public bool PassArguments { get; set; }

		public void OnEventChanged()
		{
			if (this.AssociatedObject != null)
				this.AttachHandler(this.Event);
		}

		protected override void OnSetup() => this.AttachHandler(this.Event);

		/// <summary>
		/// Attaches the handler to the event
		/// </summary>
		void AttachHandler(string eventName)
		{
			// detach old event
			if (this._oldEvent != null)
				this._oldEvent.RemoveEventHandler(this.AssociatedObject, this._handler);

			// attach new event
			if (!string.IsNullOrEmpty(eventName))
			{
				var ei = this.AssociatedObject.GetType().GetEvent(eventName);
				if (ei != null)
				{
					var mi = this.GetType().GetMethod("ExecuteCommand", BindingFlags.Instance | BindingFlags.NonPublic);
					this._handler = Delegate.CreateDelegate(ei.EventHandlerType, this, mi);
					ei.AddEventHandler(this.AssociatedObject, this._handler);
					this._oldEvent = ei; // store to detach in case the Event property changes
				}
				else
					throw new ArgumentException($"The event '{eventName}' was not found on type '{this.AssociatedObject.GetType().Name}'");
			}
		}

		/// <summary>
		/// Executes the Command
		/// </summary>
		void ExecuteCommand(object sender, EventArgs e)
		{
			object parameter = this.PassArguments ? e : null;
			if (this.Command?.CanExecute(parameter) ?? false)
				this.Command.Execute(parameter);
		}
	}
}
